@@ -18,16 +18,15 @@ class Closure < CallableSignature
   attr_reader :model
   attr_reader :enclosing_scope
 
-  def initialize(evaluator, model, scope)
+  def initialize(evaluator, model)
     @evaluator = evaluator
     @model = model
-    @enclosing_scope = scope
   end
 
   # Evaluates a closure in its enclosing scope after having matched given arguments with parameters (from left to right)
   # @api public
   def call(*args)
-    call_with_scope(@enclosing_scope, args)
+    call_with_scope(enclosing_scope, args)
   end
 
   # This method makes a Closure compatible with a Dispatch. This is used when the closure is wrapped in a Function
@@ -35,7 +34,7 @@ class Closure < CallableSignature
   # checks of the argument type/arity validity).
   # @api private
   def invoke(instance, calling_scope, args, &block)
-    @enclosing_scope.with_global_scope do |global_scope|
+    enclosing_scope.with_global_scope do |global_scope|
       call_with_scope(global_scope, args, &block)
     end
   end
@@ -43,8 +42,9 @@ class Closure < CallableSignature
   # Call closure with argument assignment by name
   def call_by_name(args_hash, enforce_parameters)
     if enforce_parameters
+      closure_scope = enclosing_scope
       # Push a temporary parameter scope used while resolving the parameter defaults
-      @enclosing_scope.with_parameter_scope(closure_name, parameter_names) do |param_scope|
+      closure_scope.with_parameter_scope(closure_name, parameter_names) do |param_scope|
         # Assign all non-nil values, even those that represent non-existent paramaters.
         args_hash.each { |k, v| param_scope[k] = v unless v.nil? }
         parameters.each do |p|
@@ -56,7 +56,7 @@ class Closure < CallableSignature
               # No default. Assign nil if the args_hash included it
               param_scope[name] = nil if args_hash.include?(name)
             else
-              param_scope[name] = param_scope.evaluate(name, p.value, @enclosing_scope, @evaluator)
+              param_scope[name] = param_scope.evaluate(name, p.value, closure_scope, @evaluator)
             end
           end
         end
@@ -65,7 +65,7 @@ class Closure < CallableSignature
       Types::TypeMismatchDescriber.validate_parameters(closure_name, params_struct, args_hash)
     end
 
-    @evaluator.evaluate_block_with_bindings(@enclosing_scope, args_hash, @model.body)
+    @evaluator.evaluate_block_with_bindings(enclosing_scope, args_hash, @model.body)
   end
 
   def parameters
@@ -113,15 +113,38 @@ class Closure < CallableSignature
     CLOSURE_NAME
   end
 
+  class Dynamic < Closure
+    def initialize(evaluator, model, scope)
+      @enclosing_scope = scope
+      super(evaluator, model)
+    end
+
+    def enclosing_scope
+      @enclosing_scope
+    end
+  end
+
   class Named < Closure
-    def initialize(name, evaluator, model, scope)
+    def initialize(name, evaluator, model)
       @name = name
-      super(evaluator, model, scope)
+      super(evaluator, model)
     end
 
     def closure_name
       @name
     end
+
+    # The assigned enclosing scope, or global scope if enclosing scope was initialized to nil
+    #
+    def enclosing_scope
+      # Named closures are typically used for puppet functions and they cannot be defined
+      # in an enclosing scope as they are cashed and reused. They need to bind to the
+      # global scope at time of use rather at time of definition.
+      # Unnamed closures are always a runtime construct, they are never bound by a loader
+      # and are thus garbage collected at end of a compilation.
+      #
+      Puppet.lookup(:global_scope) { {} }
+    end
   end
 
   private
@@ -196,8 +219,10 @@ class Closure < CallableSignature
     types = []
     range = [0, 0]
     in_optional_parameters = false
+    closure_scope = enclosing_scope
+
     parameters.each do |param|
-      type, param_range = create_param_type(param)
+      type, param_range = create_param_type(param, closure_scope)
 
       types << type
 
@@ -221,9 +246,10 @@ class Closure < CallableSignature
   def create_params_struct
     type_factory = Types::TypeFactory
     members = {}
+    closure_scope = enclosing_scope
 
     parameters.each do |param|
-      arg_type, param_range = create_param_type(param)
+      arg_type, param_range = create_param_type(param, closure_scope)
       key_type = type_factory.string(nil, param.name.to_s)
       key_type = type_factory.optional(key_type) unless param.value.nil?
       members[key_type] = arg_type
@@ -231,9 +257,9 @@ class Closure < CallableSignature
     type_factory.struct(members)
   end
 
-  def create_param_type(param)
+  def create_param_type(param, closure_scope)
     type = if param.type_expr
-             @evaluator.evaluate(param.type_expr, @enclosing_scope)
+             @evaluator.evaluate(param.type_expr, closure_scope)
            else
              Types::PAnyType::DEFAULT
            end
